@@ -24,15 +24,33 @@ enum cursor_hover_state {
   HOVER_PATROL
 };
 
-/* Selecting unit from a stack without popup. */
-enum quickselect_type {
-  SELECT_POPUP = 0, SELECT_SEA, SELECT_LAND
+enum cursor_action_state {
+  CURSOR_ACTION_DEFAULT,
+  CURSOR_ACTION_GOTO,
+  CURSOR_ACTION_SELECT,
+  CURSOR_ACTION_INVALID,
+  CURSOR_ACTION_ATTACK,
+  CURSOR_ACTION_NUKE,
+  CURSOR_ACTION_PARATROOPER,
+  CURSOR_ACTION_WAIT
 };
 
-extern int hover_unit; /* unit hover_state applies to */
+/* Selecting unit from a stack without popup. */
+enum quickselect_type {
+  SELECT_POPUP = 0, SELECT_SEA, SELECT_LAND, SELECT_APPEND
+};
+
+void control_init(void);
+void control_done(void);
+void control_unit_killed(struct unit *punit);
+
+void unit_change_battlegroup(struct unit *punit, int battlegroup);
+void unit_register_battlegroup(struct unit *punit);
+
 extern enum cursor_hover_state hover_state;
+extern enum cursor_action_state action_state;
 extern enum unit_activity connect_activity;
-extern bool draw_goto_line;
+extern enum unit_orders goto_last_order;
 extern bool non_ai_unit_focus;
 
 bool can_unit_do_connect(struct unit *punit, enum unit_activity activity);
@@ -41,13 +59,15 @@ void do_move_unit(struct unit *punit, struct unit *target_unit);
 void do_unit_goto(struct tile *ptile);
 void do_unit_nuke(struct unit *punit);
 void do_unit_paradrop_to(struct unit *punit, struct tile *ptile);
-void do_unit_patrol_to(struct unit *punit, struct tile *ptile);
-void do_unit_connect(struct unit *punit, struct tile *ptile,
+void do_unit_patrol_to(struct tile *ptile);
+void do_unit_connect(struct tile *ptile,
 		     enum unit_activity activity);
 void do_map_click(struct tile *ptile, enum quickselect_type qtype);
+void control_mouse_cursor(struct tile *ptile);
 
-void set_hover_state(struct unit *punit, enum cursor_hover_state state,
-		     enum unit_activity activity);
+void set_hover_state(struct unit_list *punits, enum cursor_hover_state state,
+		     enum unit_activity connect_activity,
+		     enum unit_orders goto_last_order);
 void request_center_focus_unit(void);
 void request_move_unit_direction(struct unit *punit, int dir);
 void request_new_unit_activity(struct unit *punit, enum unit_activity act);
@@ -56,17 +76,17 @@ void request_new_unit_activity_targeted(struct unit *punit,
 					enum tile_special_type tgt);
 void request_unit_load(struct unit *pcargo, struct unit *ptransporter);
 void request_unit_unload(struct unit *pcargo);
-void request_unit_auto(struct unit *punit);
+void request_unit_autosettlers(const struct unit *punit);
 void request_unit_build_city(struct unit *punit);
 void request_unit_caravan_action(struct unit *punit, enum packet_type action);
 void request_unit_change_homecity(struct unit *punit);
 void request_unit_connect(enum unit_activity activity);
 void request_unit_disband(struct unit *punit);
 void request_unit_fortify(struct unit *punit);
-void request_unit_goto(void);
+void request_unit_goto(enum unit_orders last_order);
 void request_unit_move_done(void);
-void request_unit_nuke(struct unit *punit);
-void request_unit_paradrop(struct unit *punit);
+void request_unit_nuke(struct unit_list *punits);
+void request_unit_paradrop(struct unit_list *punits);
 void request_unit_patrol(void);
 void request_unit_pillage(struct unit *punit);
 void request_unit_sentry(struct unit *punit);
@@ -74,10 +94,14 @@ void request_unit_unload_all(struct unit *punit);
 void request_unit_airlift(struct unit *punit, struct city *pcity);
 void request_unit_return(struct unit *punit);
 void request_unit_upgrade(struct unit *punit);
-void request_unit_wait(struct unit *punit);
+void request_units_wait(struct unit_list *punits);
 void request_unit_wakeup(struct unit *punit);
+void request_unit_select_same_type(struct unit_list *punits);
 void request_diplomat_action(enum diplomat_actions action, int dipl_id,
 			     int target_id, int value);
+void request_diplomat_answer(enum diplomat_actions action, int dipl_id,
+			     int target_id, int value);
+void request_toggle_city_outlines(void);
 void request_toggle_map_grid(void);
 void request_toggle_map_borders(void);
 void request_toggle_city_names(void);
@@ -98,16 +122,26 @@ void request_toggle_fog_of_war(void);
 
 void wakeup_sentried_units(struct tile *ptile);
 
-void auto_center_on_focus_unit(void);
-void advance_unit_focus(void);
-struct unit *get_unit_in_focus(void);
+bool unit_is_in_focus(const struct unit *punit);
+struct unit *get_focus_unit_on_tile(const struct tile *ptile);
+struct unit *head_of_units_in_focus(void);
+struct unit_list *get_units_in_focus(void);
+int get_num_units_in_focus(void);
+
 void set_unit_focus(struct unit *punit);
 void set_unit_focus_and_select(struct unit *punit);
+void add_unit_focus(struct unit *punit);
+void urgent_unit_focus(struct unit *punit);
+
+void advance_unit_focus(void);
+void auto_center_on_focus_unit(void);
 void update_unit_focus(void);
+void update_unit_pix_label(struct unit_list *punitlist);
+
 struct unit *find_visible_unit(struct tile *ptile);
 void set_units_in_combat(struct unit *pattacker, struct unit *pdefender);
-void blink_active_unit(void);
-void update_unit_pix_label(struct unit *punit);
+double blink_active_unit(void);
+double blink_turn_done_button(void);
 
 void process_caravan_arrival(struct unit *punit);
 void process_diplomat_arrival(struct unit *pdiplomat, int victim_id);
@@ -130,13 +164,12 @@ void key_units_toggle(void);
 void key_focus_unit_toggle(void);
 void key_fog_of_war_toggle(void);
 void key_end_turn(void);
+void key_city_outlines_toggle(void);
 void key_map_grid_toggle(void);
 void key_map_borders_toggle(void);
-void key_quickselect(enum quickselect_type qtype);
 void key_recall_previous_focus_unit(void);
 void key_unit_move(enum direction8 gui_dir);
 void key_unit_airbase(void);
-void key_unit_auto_attack(void);
 void key_unit_auto_explore(void);
 void key_unit_auto_settle(void);
 void key_unit_build_city(void);
@@ -164,6 +197,8 @@ void key_unit_transform(void);
 void key_unit_unload_all(void);
 void key_unit_wait(void);
 void key_unit_wakeup_others(void);
+void key_unit_assign_battlegroup(int battlegroup, bool append);
+void key_unit_select_battlegroup(int battlegroup, bool append);
 
 /* don't change this unless you also put more entries in data/Freeciv */
 #define MAX_NUM_UNITS_BELOW 4
