@@ -97,6 +97,7 @@ static void popup_next_menu(GtkMenuShell *menu, gpointer data);
 
 static GtkWidget *city_center_command, *city_popup_command, *city_buy_command;
 static GtkWidget *city_production_command;
+static GtkWidget *city_total_buy_cost_label;
 
 
 static GtkWidget *change_improvements_item;
@@ -500,15 +501,20 @@ static void select_cma_callback(GtkWidget * w, gpointer data)
       controlled = cma_is_city_under_agent(pcity, &parameter);
       select = FALSE;
 
-      if (idx == CMA_CUSTOM && controlled
-          && cmafec_preset_get_index_of_parameter(&parameter) == -1) {
-        select = TRUE;
-      } else if (idx == CMA_NONE && !controlled) {
-        select = TRUE;
-      } else if (idx >= 0 && controlled &&
-        	 cm_are_parameter_equal(&parameter,
-        				cmafec_preset_get_parameter(idx))) {
-        select = TRUE;
+      if (idx == CMA_NONE) {
+        /* CMA_NONE selects not-controlled, all others require controlled */
+        if (!controlled) {
+          select = TRUE;
+        }
+      } else if (controlled) {
+        if (idx == CMA_CUSTOM) {
+          if (cmafec_preset_get_index_of_parameter(&parameter) == -1) {
+            select = TRUE;
+          }
+        } else if (cm_are_parameter_equal(&parameter,
+                                          cmafec_preset_get_parameter(idx))) {
+          select = TRUE;
+        }
       }
 
       if (select) {
@@ -525,8 +531,8 @@ static void select_cma_callback(GtkWidget * w, gpointer data)
 
 /****************************************************************
  Create the cma entries in the change menu and the select menu. The
- indices CMA_NONE (aka -1) and CMA_CUSTOM (aka -2) are
- special. CMA_NONE signifies a preset of "none" and CMA_CUSTOM a
+ indices CMA_NONE and CMA_CUSTOM are special.
+ CMA_NONE signifies a preset of "none" and CMA_CUSTOM a
  "custom" preset.
 *****************************************************************/
 static void append_cma_to_menu_item(GtkMenuItem *parent_item, bool change_cma)
@@ -815,6 +821,10 @@ static void create_city_report_dialog(bool make_modal)
       _("_Buy"), CITY_BUY);
   city_buy_command = w;
 
+  city_total_buy_cost_label = gtk_label_new(NULL);
+  gtk_box_pack_end(GTK_BOX(city_dialog_shell->action_area),
+                   city_total_buy_cost_label, FALSE, FALSE, 0);
+
   gui_dialog_add_button(city_dialog_shell,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 
@@ -1572,6 +1582,7 @@ static void popup_select_menu(GtkMenuShell *menu, gpointer data)
 static void city_selection_changed_callback(GtkTreeSelection *selection)
 {
   int n;
+  int total = 0;
 
   n = gtk_tree_selection_count_selected_rows(selection);
 
@@ -1581,11 +1592,40 @@ static void city_selection_changed_callback(GtkTreeSelection *selection)
     gtk_widget_set_sensitive(city_popup_command, FALSE);
     gtk_widget_set_sensitive(city_buy_command, FALSE);
   } else {
+    GList *rows, *p;
+    GtkTreeModel *model;
+    GtkTreePath *path;
+    GtkTreeIter iter;
+    gpointer res;
+    struct city *pcity;
+
     gtk_widget_set_sensitive(city_production_command,
 			     can_client_issue_orders());
     gtk_widget_set_sensitive(city_center_command, TRUE);
     gtk_widget_set_sensitive(city_popup_command, TRUE);
     gtk_widget_set_sensitive(city_buy_command, can_client_issue_orders());
+
+    rows = gtk_tree_selection_get_selected_rows(selection, &model);
+    for (p = rows; p != NULL; p = p->next) {
+      path = p->data;
+      if (gtk_tree_model_get_iter(model, &iter, path)) {
+        gtk_tree_model_get(model, &iter, 0, &res, -1);
+        pcity = res;
+        if (pcity != NULL) {
+          total += city_buy_cost(pcity);
+        }
+      }
+      gtk_tree_path_free(path);
+    }
+    g_list_free(rows);
+  }
+
+  if (total > 0) {
+    char buf[64];
+    my_snprintf(buf, sizeof(buf), _("Total Buy Cost: %d"), total);
+    gtk_label_set_text(GTK_LABEL(city_total_buy_cost_label), buf);
+  } else {
+    gtk_label_set_text(GTK_LABEL(city_total_buy_cost_label), NULL);
   }
 }
 
