@@ -512,6 +512,8 @@ static void set_connection_state(enum connection_state state)
     gtk_widget_grab_focus(network_password);
     break;
   case ENTER_PASSWORD_TYPE:
+
+    set_client_page(PAGE_NETWORK);
     gtk_entry_set_text(GTK_ENTRY(network_password), "");
     gtk_entry_set_text(GTK_ENTRY(network_confirm_password), "");
 
@@ -1151,7 +1153,8 @@ static void show_conn_popup(struct player *pplayer, struct connection *pconn)
 
   /* Show popup. */
   popup = gtk_message_dialog_new(NULL, 0,
-				 GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, buf);
+				 GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
+				 "%s", buf);
   gtk_window_set_title(GTK_WINDOW(popup), _("Player/conn info"));
   setup_dialog(popup, toplevel);
   g_signal_connect(popup, "response", G_CALLBACK(gtk_widget_destroy), NULL);
