@@ -259,7 +259,7 @@ int num_known_tech_with_flag(const struct player *pplayer,
 			     enum tech_flag_id flag);
 int player_get_expected_income(const struct player *pplayer);
 
-void player_limit_to_government_rates(struct player *pplayer);
+struct player_economic player_limit_to_max_rates(struct player *pplayer);
 
 struct city *find_palace(const struct player *pplayer);
 
