@@ -185,7 +185,7 @@ static int evaluate_city_name_priority(struct tile *ptile,
   terrain_type_iterate(pterrain) {
     /* Now we do the same for every available terrain. */
     goodness
-      = is_terrain_near_tile(ptile, pterrain)
+      = is_terrain_near_tile(ptile, pterrain, TRUE)
       ? nc->terrain[pterrain->index]
       : -nc->terrain[pterrain->index];
     if (goodness > 0) {
@@ -2165,7 +2165,7 @@ void sync_cities(void)
 }
 
 /**************************************************************************
-...
+  Check worker maps of all the cities of the player.
 **************************************************************************/
 void check_city_workers(struct player *pplayer)
 {
