@@ -117,6 +117,21 @@ static void init_queue(void)
   highest_cost = 0;
 }
 
+/**************************************************************************
+  Free mapqueue arrays
+**************************************************************************/
+void free_mapqueue(void)
+{
+  int i;
+
+  for (i = 0; i < MAXARRAYS; i++) {
+    if (mappos_arrays[i] != NULL) {
+      free(mappos_arrays[i]);
+      mappos_arrays[i] = NULL;
+    }
+  }
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
