@@ -490,7 +490,7 @@ enum server_events server_sniff_all_input(void)
 	  if (time(NULL) > last_noplayers + srvarg.quitidle) {
 	    save_game_auto("Lost all connections");
 	    set_meta_message_string("restarting for lack of players");
-	    freelog(LOG_NORMAL, get_meta_message_string());
+	    freelog(LOG_NORMAL, "%s", get_meta_message_string());
 	    (void) send_server_info_to_metaserver(META_INFO);
 
             set_server_state(S_S_OVER);
@@ -516,7 +516,7 @@ enum server_events server_sniff_all_input(void)
 		      "restarting in %d seconds for lack of players",
 		      srvarg.quitidle);
           set_meta_message_string((const char *)buf);
-	  freelog(LOG_NORMAL, get_meta_message_string());
+	  freelog(LOG_NORMAL, "%s", get_meta_message_string());
 	  (void) send_server_info_to_metaserver(META_INFO);
 	}
       } else {
@@ -711,7 +711,8 @@ enum server_events server_sniff_all_input(void)
       if (didget >= 1) {
         buffer[didget-1] = '\0'; /* overwrite newline character */
         didget--;
-        freelog(LOG_DEBUG, "Got line: \"%s\" (%ld, %ld)", buffer, didget, len);
+        freelog(LOG_DEBUG, "Got line: \"%s\" (%ld, %ld)", buffer,
+                (long int) didget, (long int) len);
       }
 #else  /* HAVE_GETLINE */
       buffer = malloc(BUF_SIZE + 1);
@@ -1014,6 +1015,9 @@ static void start_processing_request(struct connection *pconn,
 **************************************************************************/
 static void finish_processing_request(struct connection *pconn)
 {
+  if (!pconn || !pconn->used) {
+    return;
+  }
   assert(pconn->server.currently_processed_request_id);
   freelog(LOG_DEBUG, "finish processing packet %d from connection %d",
 	  pconn->server.currently_processed_request_id, pconn->id);
@@ -1132,7 +1136,11 @@ static void get_lanserver_announcement(void)
 ********************************************************************/
 static void send_lanserver_response(void)
 {
+#ifndef HAVE_WINSOCK
   unsigned char buffer[MAX_LEN_PACKET];
+#else  /* HAVE_WINSOCK */
+  char buffer[MAX_LEN_PACKET];
+#endif /* HAVE_WINSOCK */
   char hostname[512];
   char port[256];
   char version[256];
